package com.btc.ep.base.dal.internal;

import java.util.Collection;
import java.util.HashSet;

import javax.persistence.EntityTransaction;

import com.btc.ep.base.bl.ModelElement;
import com.btc.ep.base.bl.impl.ModelElementImpl;
import com.btc.ep.base.dal.impl.BaseRepository;
import com.btc.ep.core.Services;
import com.btc.ep.dal.persist.PersistenceManager;
import com.btc.ep.dal.persist.impl.PersistenceService;

public class BaseRepositoryImpl implements BaseRepository<ModelElement> {

    private PersistenceManager persistenceManager;

    public void setPersistenceManager(PersistenceManager persistenceManager) {
        this.persistenceManager = persistenceManager;
    }

    private PersistenceManager getPersistenceManager() {
        if (persistenceManager == null) {
            persistenceManager = Services.get(PersistenceManager.class);
        }
        return persistenceManager;
    }

    private PersistenceService getPersistenceService() {
        return getPersistenceManager().getPersistenceService();
    }

    @Override
    public ModelElement find(String uid) {
        return getPersistenceService().find(ModelElement.class, uid);
    }

    public ModelElement load(String uid) {
        return getPersistenceService().find(ModelElement.class, uid);
    }

    @Override
    public <E extends ModelElementImpl> Collection<E> findAll(Class<E> clazz) {
        return getPersistenceService().getModelElementsByQuery("select m from " + clazz.getSimpleName() + " m", clazz);
    }

    @Override
    public ModelElement update(ModelElement dmo) {

        EntityTransaction transaction = getPersistenceService().getTransaction();

        ModelElement attachedDmo = getPersistenceService().update(dmo);

        if (transaction != null) {
            transaction.commit();
        }

        return attachedDmo;
    }

    @Override
    public Collection<ModelElement> update(Collection<ModelElement> dmos) {

        Collection<ModelElement> attachedDMOs = new HashSet<ModelElement>();
        EntityTransaction transaction = getPersistenceService().getTransaction();

        for (ModelElement dmo : dmos) {
            ModelElement attachedDmo = getPersistenceService().update(dmo);
            attachedDMOs.add(attachedDmo);
        }

        if (transaction != null) {
            transaction.commit();
        }
        return attachedDMOs;
    }

    @Override
    public void remove(ModelElement dmo) {

        EntityTransaction transaction = getPersistenceService().getTransaction();

        getPersistenceService().remove(dmo);

        if (transaction != null) {
            transaction.commit();
        }
    }

    @Override
    public void remove(Collection<ModelElement> dmos) {

        EntityTransaction transaction = getPersistenceService().getTransaction();

        for (ModelElement dmo : dmos) {
            getPersistenceService().remove(dmo);
        }

        if (transaction != null) {
            transaction.commit();
        }
    }
    
    @Override
    public void removeImpl(Collection<? extends ModelElementImpl> dmos) {
        
        for (ModelElement dmo : dmos) {
            getPersistenceService().remove(dmo);
        }
    }

    @Override
    public boolean save() {
    	getPersistenceService().commit();
    	return true;
    }

    @Override
    public boolean revert() {
        //FIXME does it works?

        EntityTransaction transaction = getPersistenceService().getTransaction();

        if (transaction != null) {
            transaction.rollback();
            persistenceManager.clear();
            return true;
        }

        return false;

    }

    @Override
    public boolean clearDatabase() {
        getPersistenceService().updateQuery("DELETE FROM Object"); // delete whole database
        return true;
    }

}
