package com.btc.ep.dal.persist.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;
import javax.persistence.FlushModeType;
import javax.persistence.Query;
import javax.persistence.TypedQuery;

import com.btc.ep.base.bl.ModelElement;

public class PersistenceService {

    protected EntityManager entityManager;

    public PersistenceService(EntityManager entityManager) {
        this.entityManager = entityManager;
        entityManager.setFlushMode(FlushModeType.AUTO);
        EntityTransaction transaction = entityManager.getTransaction();
        transaction.begin();
    }

    public void close() {
        EntityTransaction transaction = entityManager.getTransaction();
        try {
            transaction.rollback();
        } finally {
            this.entityManager.close();
        }
    }

    public void clear() {
        this.entityManager.clear();
    }

    @Deprecated
    public <T extends ModelElement> T create(T obj) {
        persist(obj);
        commit();
        return obj;
    }

    public <T extends ModelElement> void persist(T obj) {
        EntityManager em = getEntityManager();
        em.persist(obj);
    }

    public <T extends ModelElement> T update(T obj) {
        EntityManager em = getEntityManager();
        T aobj = em.merge(obj);
        return aobj;
    }

    public <T extends ModelElement> void refresh(T obj) {
        EntityManager em = getEntityManager();
        em.refresh(obj);
    }

    public void commit() {
        EntityManager em = getEntityManager();
        EntityTransaction transaction = em.getTransaction();
        try {
            transaction.commit();
        } catch (Exception ex) {
            throw ex;
        } finally {
            transaction.begin();
        }
    }

    public void rollback() {
        EntityManager em = getEntityManager();
        EntityTransaction transaction = em.getTransaction();
        transaction.rollback();
        transaction.begin();
    }

    public void flush() {
        EntityManager em = getEntityManager();
        em.flush();
    }
    
    public EntityTransaction getTransaction() {
        
        EntityTransaction transaction = getEntityManager().getTransaction();
        
        if (transaction.isActive()) {
            return null;
        } else {
            transaction.begin();
            return transaction;
        }
    }

    public <T extends ModelElement> void remove(Class<T> clazz, String uid) {
        EntityManager em = getEntityManager();
        T obj = em.find(clazz, uid);
        em.remove(obj);
    }
    
    public <T extends ModelElement> void remove(T obj) {
        EntityManager em = getEntityManager();
        em.remove(obj);
    }

    public <T extends ModelElement> boolean contains(T obj) {
        EntityManager em = getEntityManager();
        boolean res = em.contains(obj);
        return res;
    }

    public <T extends ModelElement> T get(Class<T> clazz, Object uid) {
        EntityManager em = getEntityManager();
        return em.find(clazz, uid);
    }

    public <T extends ModelElement> T find(Class<T> clazz, String uid) {

        Map<String, Object> map = new HashMap<>();
        map.put("objectdb.result-fetch", "LAZY");
        EntityManager em = getEntityManager();
        return em.find(clazz, uid, map);
    }

    public <T extends ModelElement> T getReference(Class<T> clazz, String uid) {

        Map<String, Object> map = new HashMap<>();
        map.put("objectdb.result-fetch", "LAZY");
        EntityManager em = getEntityManager();
        return em.getReference(clazz, uid);
    }

    public <T extends ModelElement> T getModelElementByNamedQuery(String query, Class<T> clazz, Object... args) {
        EntityManager em = getEntityManager();
        TypedQuery<T> q = em.createNamedQuery(query, clazz);
        for (int i = 0; i < args.length; i++) {
            Object arg = args[i];
            if (arg instanceof String) {
                arg = asLike((String)arg);
            }
            q.setParameter(i + 1, arg);
        }
        return q.getSingleResult();
    }

    public <T extends ModelElement> List<T> getModelElementsByNamedQuery(String query, Class<T> clazz,
        Object... args) {
        EntityManager em = getEntityManager();
        TypedQuery<T> q = em.createNamedQuery(query, clazz);
        q.setFlushMode(FlushModeType.AUTO);
        for (int i = 0; i < args.length; i++) {
            Object arg = args[i];
            if (arg instanceof String) {
                arg = asLike((String)arg);
            }
            q.setParameter(i + 1, arg);
        }
        return q.getResultList();
    }

    public <T extends ModelElement> T getModelElementByQuery(String query, Class<T> clazz, Object... args) {
        EntityManager em = getEntityManager();
        TypedQuery<T> q = em.createQuery(query, clazz);
        for (int i = 0; i < args.length; i++) {
            Object arg = args[i];
            if (arg instanceof String) {
                arg = asLike((String)arg);
            }
            q.setParameter(i + 1, arg);
        }
        return q.getSingleResult();
    }

    public <T extends ModelElement> List<T> getModelElementsByQuery(String query, Class<T> clazz, Object... args) {
        EntityManager em = getEntityManager();
        TypedQuery<T> q = em.createQuery(query, clazz);
        q.setHint("objectdb.result-fetch", "LAZY");
        for (int i = 0; i < args.length; i++) {
            Object arg = args[i];
            if (arg instanceof String) {
                arg = asLike((String)arg);
            }
            q.setParameter(i + 1, arg);
        }
        return q.getResultList();
    }

    public <T> T getSingleResultByNamedQuery(String queryName, Class<T> clazz, Object... args) {
        EntityManager em = getEntityManager();
        TypedQuery<T> q = em.createNamedQuery(queryName, clazz);
        for (int i = 0; i < args.length; i++) {
            Object arg = args[i];
            if (arg instanceof String) {
                arg = asLike((String)arg);
            }
            q.setParameter(i + 1, arg);
        }
        return q.getSingleResult();
    }

    public <T> T getSingleResultByQuery(String query, Class<T> clazz, Object... args) {
        EntityManager em = getEntityManager();
        TypedQuery<T> q = em.createQuery(query, clazz);
        for (int i = 0; i < args.length; i++) {
            Object arg = args[i];
            if (arg instanceof String) {
                arg = asLike((String)arg);
            }
            q.setParameter(i + 1, arg);
        }
        return q.getSingleResult();
    }

    public <T> T getSpecialSingleResultByQuery(String query, Class<T> clazz, Object... args) {
        EntityManager em = getEntityManager();
        TypedQuery<T> q = em.createQuery(query, clazz);
        
        q.setHint("objectdb.result-fetch", "LAZY");
        for (int i = 0; i < args.length; i++) {
            Object arg = args[i];
            if (arg instanceof String) {
                arg = asLike((String)arg);
            }
            q.setParameter(i + 1, arg);
        }
        return q.getSingleResult();
    }

    public <T> List<T> getResultListByQuery(String query, Class<T> clazz, Object... args) {
        EntityManager em = getEntityManager();
        TypedQuery<T> q = em.createQuery(query, clazz);
        for (int i = 0; i < args.length; i++) {
            Object arg = args[i];
            if (arg instanceof String) {
                arg = asLike((String)arg);
            }
            q.setParameter(i + 1, arg);
        }
        return q.getResultList();
    }

    public void updateByNamedQuery(String queryName, Object... args) {

        EntityManager em = getEntityManager();
        Query query = em.createNamedQuery(queryName);
        query.executeUpdate();
    }

    public void updateQuery(String query, Object... args) {

        EntityManager em = getEntityManager();
        Query q = em.createQuery(query);
        q.executeUpdate();
    }

    private EntityManager getEntityManager() {
        return this.entityManager;
    }

    private String asLike(String name) {
        String al = name.replace('*', '%');
        return al;
    }

}
